package gregapi.item;

import static gregapi.data.CS.F;
import static gregapi.data.CS.RANDOM;
import static gregapi.data.CS.T;
import forestry.api.arboriculture.IToolGrafter;
import gregapi.GT_API;
import gregapi.code.ArrayListNoNulls;
import gregapi.code.ItemStackSet;
import gregapi.data.CS.ModIDs;
import gregapi.data.LH;
import gregapi.data.MT;
import gregapi.data.TC.TC_AspectStack;
import gregapi.enchants.Enchantment_Radioactivity;
import gregapi.lang.LanguageHandler;
import gregapi.old.Textures;
import gregapi.old.interfaces.IDamagableItem;
import gregapi.old.interfaces.IToolStats;
import gregapi.oredict.OreDictMaterial;
import gregapi.util.OM;
import gregapi.util.UT;
import gregapi.util.UT.Enchantments;
import ic2.api.item.IBoxable;

import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;
import java.util.Map.Entry;

import mods.railcraft.api.core.items.IToolCrowbar;
import net.minecraft.block.Block;
import net.minecraft.client.renderer.texture.IIconRegister;
import net.minecraft.creativetab.CreativeTabs;
import net.minecraft.enchantment.Enchantment;
import net.minecraft.enchantment.EnchantmentHelper;
import net.minecraft.entity.Entity;
import net.minecraft.entity.EntityLivingBase;
import net.minecraft.entity.SharedMonsterAttributes;
import net.minecraft.entity.item.EntityMinecart;
import net.minecraft.entity.player.EntityPlayer;
import net.minecraft.item.EnumAction;
import net.minecraft.item.Item;
import net.minecraft.item.ItemStack;
import net.minecraft.nbt.NBTTagCompound;
import net.minecraft.potion.Potion;
import net.minecraft.stats.AchievementList;
import net.minecraft.stats.StatList;
import net.minecraft.util.IIcon;
import net.minecraft.util.MathHelper;
import net.minecraft.world.World;
import net.minecraftforge.event.entity.player.PlayerEvent;
import net.minecraftforge.event.world.BlockEvent;
import buildcraft.api.tools.IToolWrench;
import cpw.mods.fml.common.Optional;
import cpw.mods.fml.relauncher.Side;
import cpw.mods.fml.relauncher.SideOnly;

/**
 * @author Gregorius Techneticies
 * 
 * This is an example on how you can create a Tool ItemStack, in this case a Bismuth Wrench:
 * CS.ToolIDs.sMetaTool.getToolWithStats(CS.ToolIDs.WRENCH, 1, MT.Bismuth, MT.Bismuth, null);
 */
@Optional.InterfaceList(value = {
  @Optional.Interface(iface = "forestry.api.arboriculture.IToolGrafter", modid = ModIDs.FR)
, @Optional.Interface(iface = "mods.railcraft.api.core.items.IToolCrowbar", modid = ModIDs.RC)
, @Optional.Interface(iface = "buildcraft.api.tools.IToolWrench", modid = ModIDs.BC)
, @Optional.Interface(iface = "ic2.api.item.IBoxable", modid = ModIDs.IC2)
})
public class MultiItemTool extends MultiItem implements IDamagableItem, IToolGrafter, IToolCrowbar, IToolWrench, IBoxable {
	public final HashMap<Short, IToolStats> mToolStats = new HashMap<Short, IToolStats>();
	
	/**
	 * Creates the Item using these Parameters.
	 * @param aUnlocalized The unlocalised Name of this Item. DO NOT START YOUR UNLOCALISED NAME WITH "gt."!!!
	 */
	public MultiItemTool(String aModID, String aUnlocalized) {
		super(aModID, aUnlocalized);
        setMaxStackSize(1);
	}
	
	/**
	 * This adds a Custom Item to the ending Range.
	 * @param aID The Id of the assigned Tool Class [0 - 32765] (only even Numbers allowed! Uneven ID's are empty electric Items)
	 * @param aEnglish The Default Localized Name of the created Item
	 * @param aToolTip The Default ToolTip of the created Item, you can also insert null for having no ToolTip
	 * @param aToolStats The Food Value of this Item. Can be null as well.
	 * @param aRandomParameters The OreDict Names you want to give the Item. Also used to assign Thaumcraft Aspects.
	 * @return An ItemStack containing the newly created Item, but without specific Stats.
	 */
	public final ItemStack addTool(int aID, String aEnglish, String aToolTip, IToolStats aToolStats, Object... aRandomParameters) {
		if (aToolTip == null) aToolTip = "";
		if (aID >= 0 && aID < 32766 && aID % 2 == 0) {
			LanguageHandler.add(getUnlocalizedName() + "." +  aID    + ".name"		, aEnglish);
			LanguageHandler.add(getUnlocalizedName() + "." +  aID    + ".tooltip"	, aToolTip);
			LanguageHandler.add(getUnlocalizedName() + "." + (aID+1) + ".name"		, aEnglish + " (Empty)");
			LanguageHandler.add(getUnlocalizedName() + "." + (aID+1) + ".tooltip"	, "You need to recharge it");
			mToolStats.put((short) aID   , aToolStats);
			mToolStats.put((short)(aID+1), aToolStats);
			aToolStats.onStatsAddedToTool(this, aID);
			ItemStack rStack = new ItemStack(this, 1, aID);
			List<TC_AspectStack> tAspects = new ArrayListNoNulls<TC_AspectStack>();
			for (Object tRandomParameter : aRandomParameters) {
				if (tRandomParameter instanceof TC_AspectStack)
					((TC_AspectStack)tRandomParameter).addToAspectList(tAspects);
				else if (tRandomParameter instanceof ItemStackSet)
					((ItemStackSet)tRandomParameter).add(rStack.copy());
				else
					OM.reg(tRandomParameter, rStack);
			}
			if (GT_API.sCompatTC != null) GT_API.sCompatTC.registerThaumcraftAspectsToItem(rStack, tAspects, false);
			return rStack;
		}
		return null;
	}
	
	/**
	 * This Function gets an ItemStack Version of this Tool
	 * @param aToolID the ID of the Tool Class
	 * @param aAmount Amount of Items (well normally you only need 1)
	 * @param aPrimaryMaterial Primary Material of this Tool
	 * @param aSecondaryMaterial Secondary (Rod/Handle) Material of this Tool
	 * @param aElectricArray The Electric Stats of this Tool (or null if not electric)
	 */
	public final ItemStack getToolWithStats(int aToolID, int aAmount, OreDictMaterial aPrimaryMaterial, OreDictMaterial aSecondaryMaterial, long[] aElectricArray) {
		ItemStack rStack = new ItemStack(this, aAmount, aToolID);
		IToolStats tToolStats = getToolStats(rStack);
		if (tToolStats != null) {
			NBTTagCompound tMainNBT = new NBTTagCompound(), tToolNBT = new NBTTagCompound();
			if (aPrimaryMaterial != null) {
				if (aPrimaryMaterial.mID > 0) tToolNBT.setShort("a", aPrimaryMaterial.mID); else tToolNBT.setString("b", aPrimaryMaterial.toString());
				UT.NBT.setNumber(tToolNBT, "j", 100L*(long)(aPrimaryMaterial.mToolDurability * tToolStats.getMaxDurabilityMultiplier()));
			}
			if (aSecondaryMaterial != null) {
				if (aSecondaryMaterial.mID > 0) tToolNBT.setShort("c", aSecondaryMaterial.mID); else tToolNBT.setString("d", aSecondaryMaterial.toString());
			}
			if (aElectricArray != null) {
				tToolNBT.setBoolean("e", true);
				UT.NBT.setNumber(tToolNBT, "f", aElectricArray[0]);
				UT.NBT.setNumber(tToolNBT, "g", aElectricArray[1]);
				UT.NBT.setNumber(tToolNBT, "h", aElectricArray[2]);
				UT.NBT.setNumber(tToolNBT, "i", aElectricArray[3]);
			}
			
			tMainNBT.setTag("GT.ToolStats", tToolNBT);
			rStack.setTagCompound(tMainNBT);
		}
		isItemStackUsable(rStack);
		return rStack;
	}
	
	/**
	 * Called by the Block Harvesting Event within the GT_Proxy
	 */
	public void onHarvestBlockEvent(ArrayList<ItemStack> aDrops, ItemStack aStack, EntityPlayer aPlayer, Block aBlock, int aX, int aY, int aZ, byte aMetaData, int aFortune, boolean aSilkTouch, BlockEvent.HarvestDropsEvent aEvent) {
    	IToolStats tStats = getToolStats(aStack);
    	if (isItemStackUsable(aStack) && getDigSpeed(aStack, aBlock, aMetaData) > 0.0F) doDamage(aStack, tStats.convertBlockDrops(aDrops, aStack, aPlayer, aBlock, (getToolMaxDamage(aStack) - getToolDamage(aStack)) / tStats.getToolDamagePerDropConversion(), aX, aY, aZ, aMetaData, aFortune, aSilkTouch, aEvent) * tStats.getToolDamagePerDropConversion());
    }
	
	public float onBlockBreakSpeedEvent(float aDefault, ItemStack aStack, EntityPlayer aPlayer, Block aBlock, int aX, int aY, int aZ, byte aMetaData, PlayerEvent.BreakSpeed aEvent) {
    	IToolStats tStats = getToolStats(aStack);
    	return tStats == null ? aDefault : tStats.getMiningSpeed(aBlock, aMetaData, aDefault, aPlayer, aPlayer.worldObj, aX, aY, aZ);
    }
	
    @Override
    public boolean onLeftClickEntity(ItemStack aStack, EntityPlayer aPlayer, Entity aEntity) {
    	IToolStats tStats = getToolStats(aStack);
    	if (tStats == null || !isItemStackUsable(aStack)) return true;
        UT.Sounds.play(tStats.getEntityHitSound(), 1, 1.0F, aEntity);
        if (super.onLeftClickEntity(aStack, aPlayer, aEntity)) return true;
        if (aEntity.canAttackWithItem() && !aEntity.hitByEntity(aPlayer)) {
            float tMagicDamage = tStats.getMagicDamageAgainstEntity(aEntity instanceof EntityLivingBase?EnchantmentHelper.getEnchantmentModifierLiving(aPlayer, (EntityLivingBase)aEntity):0.0F, aEntity, aStack, aPlayer), tDamage = tStats.getNormalDamageAgainstEntity((float)aPlayer.getEntityAttribute(SharedMonsterAttributes.attackDamage).getAttributeValue() + getToolCombatDamage(aStack), aEntity, aStack, aPlayer);
            if (tDamage + tMagicDamage > 0.0F) {
                boolean tCriticalHit = aPlayer.fallDistance > 0.0F && !aPlayer.onGround && !aPlayer.isOnLadder() && !aPlayer.isInWater() && !aPlayer.isPotionActive(Potion.blindness) && aPlayer.ridingEntity == null && aEntity instanceof EntityLivingBase;
                if (tCriticalHit && tDamage > 0.0F) tDamage *= 1.5F;
                tDamage += tMagicDamage;
                if (aEntity.attackEntityFrom(tStats.getDamageSource(aPlayer, aEntity), tDamage)) {
                	if (aEntity instanceof EntityLivingBase) aEntity.setFire(EnchantmentHelper.getFireAspectModifier(aPlayer) * 4);
                    int tKnockcack = (aPlayer.isSprinting()?1:0) + (aEntity instanceof EntityLivingBase?EnchantmentHelper.getKnockbackModifier(aPlayer, (EntityLivingBase)aEntity):0);
                    if (tKnockcack > 0) {
                        aEntity.addVelocity(-MathHelper.sin(aPlayer.rotationYaw * (float)Math.PI / 180.0F) * tKnockcack * 0.5F, 0.1D, MathHelper.cos(aPlayer.rotationYaw * (float)Math.PI / 180.0F) * tKnockcack * 0.5F);
                        aPlayer.motionX *= 0.6D;
                        aPlayer.motionZ *= 0.6D;
                        aPlayer.setSprinting(false);
                    }
                    if (tCriticalHit) aPlayer.onCriticalHit(aEntity);
                    if (tMagicDamage > 0.0F) aPlayer.onEnchantmentCritical(aEntity);
                    if (tDamage >= 18.0F) aPlayer.triggerAchievement(AchievementList.overkill);
                    aPlayer.setLastAttacker(aEntity);
                    if (aEntity instanceof EntityLivingBase) Enchantments.applyBullshitA((EntityLivingBase)aEntity, aPlayer, aStack);
                    Enchantments.applyBullshitB(aPlayer, aEntity, aStack);
                    if (aEntity instanceof EntityLivingBase) aPlayer.addStat(StatList.damageDealtStat, Math.round(tDamage * 10.0F));
                    aEntity.hurtResistantTime = Math.max(1, tStats.getHurtResistanceTime(aEntity.hurtResistantTime, aEntity));
                    aPlayer.addExhaustion(tStats.getExhaustionPerAttack(aEntity));
                    doDamage(aStack, tStats.getToolDamagePerEntityAttack());
                }
            }
        }
        if (aStack.stackSize <= 0) aPlayer.destroyCurrentEquippedItem();
        return true;
    }
    
    @Override
	public ItemStack onItemRightClick(ItemStack aStack, World aWorld, EntityPlayer aPlayer) {
		IToolStats tStats = getToolStats(aStack);
    	if (tStats != null && tStats.canBlock()) aPlayer.setItemInUse(aStack, 72000);
		return super.onItemRightClick(aStack, aWorld, aPlayer);
    }
    
    @Override
    public final int getMaxItemUseDuration(ItemStack aStack) {
        return 72000;
    }
    
    @Override
	public final EnumAction getItemUseAction(ItemStack aStack) {
		IToolStats tStats = getToolStats(aStack);
		if (tStats != null && tStats.canBlock()) return EnumAction.block;
        return EnumAction.none;
    }
    
	@Override
    @SideOnly(Side.CLIENT)
    public final void getSubItems(Item var1, CreativeTabs aCreativeTab, List aList) {
		for (int i = 0; i < 32766; i+=2) if (getToolStats(new ItemStack(this, 1, i)) != null) {
			ItemStack tStack = new ItemStack(this, 1, i);
			isItemStackUsable(tStack);
			aList.add(tStack);
		}
    }
	
	@Override
    public void addAdditionalToolTips(List aList, ItemStack aStack, boolean aF3_H) {
		long tMaxDamage = getToolMaxDamage(aStack);
		OreDictMaterial tMaterial = getPrimaryMaterial(aStack);
		IToolStats tStats = getToolStats(aStack);
		int tOffset = getElectricStats(aStack) != null ? 2 : 1;
		if (tStats != null) {
			aList.add(tOffset + 0, LH.Chat.WHITE + "Durability: " + LH.Chat.GREEN + (tMaxDamage - getToolDamage(aStack)) + " / " + tMaxDamage + LH.Chat.GRAY);
			aList.add(tOffset + 1, LH.Chat.WHITE + tMaterial.getLocal() + LH.Chat.YELLOW + " lvl " + getHarvestLevel(aStack, "") + LH.Chat.GRAY);
			aList.add(tOffset + 2, LH.Chat.WHITE + "Attack Damage: " + LH.Chat.BLUE + getToolCombatDamage(aStack) + LH.Chat.GRAY);
			aList.add(tOffset + 3, LH.Chat.WHITE + "Mining Speed: " + LH.Chat.PINK + Math.max(Float.MIN_NORMAL, tStats.getSpeedMultiplier() * getPrimaryMaterial(aStack).mToolSpeed) + LH.Chat.GRAY);
		}
	}
	
	public static final OreDictMaterial getPrimaryMaterial(ItemStack aStack) {
		NBTTagCompound aNBT = aStack.getTagCompound();
		if (aNBT != null) {
			aNBT = aNBT.getCompoundTag("GT.ToolStats");
			if (aNBT != null) {
				if (aNBT.hasKey("a")) return OreDictMaterial.MATERIAL_ARRAY[aNBT.getShort("a")];
				if (aNBT.hasKey("b")) return OreDictMaterial.get(aNBT.getString("b"));
				return OreDictMaterial.get(aNBT.getString("PrimaryMaterial"));
			}
		}
		return MT.NULL;
	}
	
	public static final OreDictMaterial getSecondaryMaterial(ItemStack aStack) {
		NBTTagCompound aNBT = aStack.getTagCompound();
		if (aNBT != null) {
			aNBT = aNBT.getCompoundTag("GT.ToolStats");
			if (aNBT != null) {
				if (aNBT.hasKey("c")) return OreDictMaterial.MATERIAL_ARRAY[aNBT.getShort("c")];
				if (aNBT.hasKey("d")) return OreDictMaterial.get(aNBT.getString("d"));
				return OreDictMaterial.get(aNBT.getString("SecondaryMaterial"));
			}
		}
		return MT.NULL;
	}
	
	@Override
	public Long[] getElectricStats(ItemStack aStack) {
		NBTTagCompound aNBT = aStack.getTagCompound();
		if (aNBT != null) {
			aNBT = aNBT.getCompoundTag("GT.ToolStats");
			if (aNBT != null) {
				if (aNBT.getBoolean("e")) return new Long[] {aNBT.getLong("f"), aNBT.getLong("g"), aNBT.getLong("h"), aNBT.getLong("i")};
				if (aNBT.getBoolean("Electric")) return new Long[] {aNBT.getLong("MaxCharge"), aNBT.getLong("Voltage"), aNBT.getLong("Tier"), aNBT.getLong("SpecialData")};
			}
		}
		return null;
	}
	
	public float getToolCombatDamage(ItemStack aStack) {
		IToolStats tStats = getToolStats(aStack);
		if (tStats == null) return 0;
		return tStats.getBaseDamage() + getPrimaryMaterial(aStack).mToolQuality;
	}
	
	public static final long getToolMaxDamage(ItemStack aStack) {
		NBTTagCompound aNBT = aStack.getTagCompound();
		if (aNBT != null) {
			aNBT = aNBT.getCompoundTag("GT.ToolStats");
			if (aNBT != null) {
				if (aNBT.hasKey("j")) return aNBT.getLong("j");
				return aNBT.getLong("MaxDamage");
			}
		}
		return 0;
	}
	
	public static final long getToolDamage(ItemStack aStack) {
		NBTTagCompound aNBT = aStack.getTagCompound();
		if (aNBT != null) {
			aNBT = aNBT.getCompoundTag("GT.ToolStats");
			if (aNBT != null) {
				if (aNBT.hasKey("k")) return aNBT.getLong("k");
				return aNBT.getLong("Damage");
			}
		}
		return 0;
	}
	
	public static final boolean setToolDamage(ItemStack aStack, long aDamage) {
		NBTTagCompound aNBT = aStack.getTagCompound();
		if (aNBT != null) {
			aNBT = aNBT.getCompoundTag("GT.ToolStats");
			if (aNBT != null) {
				UT.NBT.setNumber(aNBT, "k", aDamage);
				return true;
			}
		}
		return false;
	}
	
	@Override
	public final boolean doDamageToItem(ItemStack aStack, int aVanillaDamage) {
		return doDamage(aStack, aVanillaDamage * 100);
	}
	
	public final boolean doDamage(ItemStack aStack, long aAmount) {
		if (!isItemStackUsable(aStack)) return false;
		Long[] tElectric = getElectricStats(aStack);
		if (tElectric == null) {
			long tNewDamage = getToolDamage(aStack) + aAmount;
			setToolDamage(aStack, tNewDamage);
			if (tNewDamage >= getToolMaxDamage(aStack)) {
				IToolStats tStats = getToolStats(aStack);
				if (tStats != null) UT.Sounds.play(tStats.getBreakingSound(), 1, 1.0F);
				if (tStats == null || UT.Stacks.set(aStack, tStats.getBrokenItem(aStack)) == null) {
					if (aStack.stackSize > 0) aStack.stackSize--;
				}
			}
			return true;
		}
		if (use(aStack, (int)aAmount, null)) {
			if (RANDOM.nextInt(25) == 0) {
				long tNewDamage = getToolDamage(aStack) + aAmount;
				setToolDamage(aStack, tNewDamage);
				if (tNewDamage >= getToolMaxDamage(aStack)) {
					IToolStats tStats = getToolStats(aStack);
					if (tStats != null) UT.Sounds.play(tStats.getBreakingSound(), 1, 1.0F);
					if (tStats == null || UT.Stacks.set(aStack, tStats.getBrokenItem(aStack)) == null) {
						if (aStack.stackSize > 0) aStack.stackSize--;
					}
				}
			}
			return true;
		}
		return false;
	}
	
    @Override
    public float getDigSpeed(ItemStack aStack, Block aBlock, int aMetaData) {
		if (!isItemStackUsable(aStack)) return 0.0F;
		IToolStats tStats = getToolStats(aStack);
		if (tStats == null || Math.max(0, getHarvestLevel(aStack, "")) < aBlock.getHarvestLevel(aMetaData)) return 0.0F;
		return tStats.getMiningSpeed(aBlock, (byte)aMetaData) * Math.max(Float.MIN_NORMAL, tStats.getSpeedMultiplier() * getPrimaryMaterial(aStack).mToolSpeed);
    }
    
	@Override
    public final boolean canHarvestBlock(Block aBlock, ItemStack aStack) {
		return getDigSpeed(aStack, aBlock, (byte)0) > 0.0F;
	}
	
	@Override
    public final int getHarvestLevel(ItemStack aStack, String aToolClass) {
		IToolStats tStats = getToolStats(aStack);
        return tStats == null ? -1 : tStats.getBaseQuality() + getPrimaryMaterial(aStack).mToolQuality; 
    }
    
	@Override
    public boolean onBlockDestroyed(ItemStack aStack, World aWorld, Block aBlock, int aX, int aY, int aZ, EntityLivingBase aPlayer) {
		if (!isItemStackUsable(aStack)) return false;
		IToolStats tStats = getToolStats(aStack);
		if (tStats == null) return false;
        UT.Sounds.play(tStats.getMiningSound(), 1, 1.0F, aX, aY, aZ);
        doDamage(aStack, (int)Math.max(1, aBlock.getBlockHardness(aWorld, aX, aY, aZ) * tStats.getToolDamagePerBlockBreak()));
        return getDigSpeed(aStack, aBlock, aWorld.getBlockMetadata(aX, aY, aZ)) > 0.0F;
    }
    
	@Override
    public final ItemStack getContainerItem(ItemStack aStack) {
		if (!isItemStackUsable(aStack)) return null;
		aStack = UT.Stacks.copyAmount(1, aStack);
		IToolStats tStats = getToolStats(aStack);
		if (tStats == null) return null;
		doDamage(aStack, tStats.getToolDamagePerContainerCraft());
		aStack = aStack.stackSize > 0 ? aStack : null;
		if (aStack == null) UT.Sounds.play(tStats.getBreakingSound(), 1, 1.0F); else UT.Sounds.play(tStats.getCraftingSound(), 1, 1.0F);
		return aStack;
    }
	
	public IToolStats getToolStats(ItemStack aStack) {
		isItemStackUsable(aStack);
		return getToolStatsInternal(aStack);
	}
	
	private IToolStats getToolStatsInternal(ItemStack aStack) {
		return aStack == null ? null : getToolStatsInternal(UT.Stacks.meta(aStack));
	}
	
	private IToolStats getToolStatsInternal(int aDamage) {
		return mToolStats.get((short)aDamage);
	}
	
	@Override
	public float getSaplingModifier(ItemStack aStack, World aWorld, EntityPlayer aPlayer, int aX, int aY, int aZ) {
		IToolStats tStats = getToolStats(aStack);
		return tStats != null && tStats.isGrafter() ? Math.min(100.0F, (1+getHarvestLevel(aStack, "")) * 20.0F) : 0.0F;
	}
	
	@Override
	public boolean canWrench(EntityPlayer aPlayer, int aX, int aY, int aZ) {
		ItemStack aStack = aPlayer.getCurrentEquippedItem();
		if (!isItemStackUsable(aStack)) return false;
		IToolStats tStats = getToolStats(aStack);
		return tStats != null && tStats.isWrench();
	}
	
	@Override
	public void wrenchUsed(EntityPlayer aPlayer, int aX, int aY, int aZ) {
		ItemStack aStack = aPlayer.getCurrentEquippedItem();
		IToolStats tStats = getToolStats(aStack);
		if (tStats != null) doDamage(aStack, 100);
	}
	
	@Override
	public boolean canWhack(EntityPlayer aPlayer, ItemStack aStack, int aX, int aY, int aZ) {
		if (!isItemStackUsable(aStack)) return false;
		IToolStats tStats = getToolStats(aStack);
		return tStats != null && tStats.isCrowbar();
	}
	
	@Override
	public void onWhack(EntityPlayer aPlayer, ItemStack aStack, int aX, int aY, int aZ) {
		IToolStats tStats = getToolStats(aStack);
		if (tStats != null) doDamage(aStack, 100);
	}
	
	@Override
	public boolean canLink(EntityPlayer aPlayer, ItemStack aStack, EntityMinecart cart) {
		if (!isItemStackUsable(aStack)) return false;
		IToolStats tStats = getToolStats(aStack);
		return tStats != null && tStats.isCrowbar();
	}
	
	@Override
	public void onLink(EntityPlayer aPlayer, ItemStack aStack, EntityMinecart cart) {
		IToolStats tStats = getToolStats(aStack);
		if (tStats != null) doDamage(aStack, tStats.getToolDamagePerEntityAttack());
	}
	
	@Override
	public boolean canBoost(EntityPlayer aPlayer, ItemStack aStack, EntityMinecart cart) {
		if (!isItemStackUsable(aStack)) return F;
		IToolStats tStats = getToolStats(aStack);
		return tStats != null && tStats.isCrowbar();
	}
	
	@Override
	public void onBoost(EntityPlayer aPlayer, ItemStack aStack, EntityMinecart cart) {
		IToolStats tStats = getToolStats(aStack);
		if (tStats != null) doDamage(aStack, tStats.getToolDamagePerEntityAttack());
	}
	
	@Override
    public void onCreated(ItemStack aStack, World aWorld, EntityPlayer aPlayer) {
		IToolStats tStats = getToolStats(aStack);
		if (tStats != null && aPlayer != null) tStats.onToolCrafted(aStack, aPlayer);
		super.onCreated(aStack, aWorld, aPlayer);
	}
    
	@Override
    public final boolean doesContainerItemLeaveCraftingGrid(ItemStack aStack) {
        return F;
    }
	
	@Override
	public final int getItemStackLimit(ItemStack aStack) {
		return 1;
	}
	
	@Override
    public boolean isFull3D() {
        return T;
    }
	
	@Override
	public boolean isItemStackUsable(ItemStack aStack) {
		IToolStats tStats = getToolStatsInternal(aStack);
		if (UT.Stacks.meta(aStack) % 2 == 1 || tStats == null) {
			NBTTagCompound aNBT = aStack.getTagCompound();
			if (aNBT != null) aNBT.removeTag("ench");
			return F;
		}
		OreDictMaterial aMaterial = getPrimaryMaterial(aStack);
		HashMap<Integer, Integer> tMap = new HashMap<Integer, Integer>(), tResult = new HashMap<Integer, Integer>();
		if (aMaterial.mEnchantmentTools != null) {
			tMap.put(aMaterial.mEnchantmentTools.effectId, (int)aMaterial.mEnchantmentToolsLevel);
			if (aMaterial.mEnchantmentTools == Enchantment.fortune)
			tMap.put(Enchantment.looting.effectId, (int)aMaterial.mEnchantmentToolsLevel);
			if (aMaterial.mEnchantmentTools == Enchantment.knockback)
			tMap.put(Enchantment.punch.effectId, (int)aMaterial.mEnchantmentToolsLevel);
			if (aMaterial.mEnchantmentTools == Enchantment.fireAspect)
			tMap.put(Enchantment.flame.effectId, (int)aMaterial.mEnchantmentToolsLevel);
		}
		Enchantment[] tEnchants = tStats.getEnchantments(aStack, aMaterial);
		int[] tLevels = tStats.getEnchantmentLevels(aStack);
		for (int i = 0; i < tEnchants.length; i++) if (tLevels[i] > 0) {
			Integer tLevel = tMap.get(tEnchants[i].effectId);
			tMap.put(tEnchants[i].effectId, tLevel == null ? tLevels[i] : tLevel == tLevels[i] ? tLevel+1 : Math.max(tLevel, tLevels[i]));
		}
		for (Entry<Integer, Integer> tEntry : tMap.entrySet()) {
			if (tEntry.getKey() == 33 || (tEntry.getKey() == 20 && tEntry.getValue() > 2) || tEntry.getKey() == Enchantment_Radioactivity.INSTANCE.effectId) tResult.put(tEntry.getKey(), tEntry.getValue()); else
			switch(Enchantment.enchantmentsList[tEntry.getKey()].type) {
			case weapon:
				if (tStats.isWeapon()) tResult.put(tEntry.getKey(), tEntry.getValue());
				break;
			case all:
				tResult.put(tEntry.getKey(), tEntry.getValue());
				break;
			case armor: case armor_feet: case armor_head: case armor_legs: case armor_torso:
				break;
			case bow:
				if (tStats.isRangedWeapon()) tResult.put(tEntry.getKey(), tEntry.getValue());
				break;
			case breakable:
				break;
			case fishing_rod:
				break;
			case digger:
				if (tStats.isMiningTool()) tResult.put(tEntry.getKey(), tEntry.getValue());
				break;
			}
		}
		EnchantmentHelper.setEnchantments(tResult, aStack);
		return true;
	}
	
	@Override
	public short getChargedMetaData(ItemStack aStack) {
		return (short)(UT.Stacks.meta(aStack) - (UT.Stacks.meta(aStack) % 2));
	}
	
	@Override
	public short getEmptyMetaData(ItemStack aStack) {
		NBTTagCompound aNBT = aStack.getTagCompound();
		if (aNBT != null) aNBT.removeTag("ench");
		return (short)(UT.Stacks.meta(aStack)+1-(UT.Stacks.meta(aStack) % 2));
	}
	
	@Override public int getItemEnchantability() {return 0;}
	@Override public boolean isBookEnchantable(ItemStack aStack, ItemStack aBook) {return false;}
	@Override public boolean getIsRepairable(ItemStack aStack, ItemStack aMaterial) {return false;}
	
	@Override
    public int getRenderPasses(int aMetaData) {
		IToolStats tStats = getToolStatsInternal(aMetaData);
		if (tStats != null) return tStats.getRenderPasses()+2;
        return 2;
    }
    
	@Override
    public boolean requiresMultipleRenderPasses() {
        return true;
    }
	
	@Override
    public void registerIcons(IIconRegister aIconRegister) {
		//
    }
	
	@Override
    public int getSpriteNumber() {
        return 1;
    }
	
	@Override
    public int getColorFromItemStack(ItemStack aStack, int aRenderPass) {
		IToolStats tStats = getToolStatsInternal(aStack);
		if (tStats != null) return UT.Code.getRGBaInt(tStats.getRGBa(aStack, aRenderPass));
		return 16777215;
    }
	
	@Override
    public IIcon getIconIndex(ItemStack aStack) {
		return getIcon(aStack, 0);
    }
	
	@Override
    public IIcon getIconFromDamage(int aMetaData) {
		return getIconIndex(new ItemStack(this, 1, aMetaData));
    }
	
	@Override
    public IIcon getIcon(ItemStack aStack, int aRenderPass) {
		return getIcon(aStack, aRenderPass, null, null, 0);
    }
	
	@Override
    public IIcon getIcon(ItemStack aStack, int aRenderPass, EntityPlayer aPlayer, ItemStack aUsedStack, int aUseRemaining) {
		IToolStats tStats = getToolStatsInternal(aStack);
		if (tStats == null) return Textures.ItemIcons.VOID.getIcon(0);
		if (aRenderPass < tStats.getRenderPasses()) {
			IIcon rIcon = tStats.getIcon(aStack, aRenderPass);
			return rIcon == null ? Textures.ItemIcons.VOID.getIcon(0) : rIcon;
		}
		if (aPlayer == null) {
			aRenderPass -= tStats.getRenderPasses();
			if (aRenderPass == 0) {
				long tDamage = MultiItemTool.getToolDamage(aStack), tMaxDamage = MultiItemTool.getToolMaxDamage(aStack);
				if (tDamage <= 0) return Textures.ItemIcons.DURABILITY_BAR[8].getIcon(0);
				if (tDamage >= tMaxDamage) return Textures.ItemIcons.DURABILITY_BAR[0].getIcon(0);
				return Textures.ItemIcons.DURABILITY_BAR[(int)Math.max(0, Math.min(7, ((tMaxDamage-tDamage)*8L) / tMaxDamage))].getIcon(0);
			}
			if (aRenderPass == 1) {
				Long[] tElectric = getElectricStats(aStack);
				if (tElectric != null && tElectric[3] < 0) {
					long tCharge = getRealCharge(aStack);
					if (tCharge <= 0) return Textures.ItemIcons.ENERGY_BAR[0].getIcon(0);
					if (tCharge >= tElectric[0]) return Textures.ItemIcons.ENERGY_BAR[8].getIcon(0);
					return Textures.ItemIcons.ENERGY_BAR[7-(int)Math.max(0, Math.min(6, ((tElectric[0]-tCharge)*7L) / tElectric[0]))].getIcon(0);
				}
			}
		}
		return Textures.ItemIcons.VOID.getIcon(0);
    }
	
    @Override
    public IIcon getIconFromDamageForRenderPass(int aMetaData, int aRenderPass) {
		return getIconFromDamage(aMetaData);
    }
    
    @Override
    public boolean hasEffect(ItemStack aStack) {
		return false;
    }
    
    @Override
    public boolean hasEffect(ItemStack aStack, int aRenderPass) {
		return false;
    }
    
	@Override
	public Long[] getFluidContainerStats(ItemStack aStack) {
		return null;
	}
	
	@Override
	public boolean canBeStoredInToolbox(ItemStack itemstack) {
		return true;
	}
}