package gregapi.render;

import static gregapi.data.CS.F;
import static gregapi.data.CS.T;
import static gregapi.data.CS.UNCOLOURED;
import gregapi.GT_API;
import gregapi.code.ArrayListNoNulls;
import gregapi.data.CS.ModIDs;

import java.util.List;

import net.minecraft.client.renderer.texture.IIconRegister;
import net.minecraft.client.renderer.texture.TextureMap;
import net.minecraft.util.IIcon;
import net.minecraft.util.ResourceLocation;

/** 
 * @author Gregorius Techneticies
 */
public class TextureSet {
	public static final List<TextureSet> INSTANCES_ITEM = new ArrayListNoNulls<TextureSet>();
	public static final List<TextureSet> INSTANCES_BLOCK = new ArrayListNoNulls<TextureSet>();
	public static final List<String> FILENAMES_ITEM = new ArrayListNoNulls<String>();
	public static final List<String> FILENAMES_BLOCK = new ArrayListNoNulls<String>();
	public final List<IIconContainer> mList = new ArrayListNoNulls<IIconContainer>();
	
	private final boolean mIsItem;
	private final String mNameSet;
	
	private TextureSet(boolean aIsItem, String aNameSet) {
		mIsItem = aIsItem;
		mNameSet = aNameSet;
	}
	
	/**
	 * Adds a new Texture Set, unless another Set with the same Name already exists.
	 * @param aModID the Mod responsible for adding the Icons (which are located in either assets\aModID.toLowerCase()\textures\items\materialicons\ or assets\aModID.toLowerCase()\textures\blocks\materialicons\).
	 * @param aIsItem true if this is an Item Icon, false if this is a Block Icon.
	 */
	public static TextureSet addTextureSet(String aModID, boolean aIsItem, String aNameSet) {
		List<TextureSet> tTextureSetList = (aIsItem?INSTANCES_ITEM:INSTANCES_BLOCK);
		List<String> tFileNameList = (aIsItem?FILENAMES_ITEM:FILENAMES_BLOCK);
		for (TextureSet tSet : tTextureSetList) if (tSet.mNameSet.equals(aNameSet)) return tSet;
		TextureSet rSet = new TextureSet(aIsItem, aNameSet);
		tTextureSetList.add(rSet);
		for (int i = 0, j = tFileNameList.size(); i < j; i++) rSet.mList.add(aIsItem?new TextureSetIconItem(aModID, aNameSet+"/"+tFileNameList.get(i)):new TextureSetIconBlock(aModID, aNameSet+"/"+tFileNameList.get(i)));
		return rSet;
	}
	
	/**
	 * Adds a new Icon Name to all Texture Sets, or just returns the Index of an already existing Set with the same File Name.
	 * @param aModID the Mod responsible for adding the Icons (which are located in either assets\aModID.toLowerCase()\textures\items\materialicons\ or assets\aModID.toLowerCase()\textures\blocks\materialicons\).
	 * @param aNameFile the Name of the File inside each Set Folder.
	 * @return the Index of this Icon inside the Set Lists.
	 */
	public static int addToAll(String aModID, boolean aIsItem, String aNameFile) {
		List<String> tFileNameList = (aIsItem?FILENAMES_ITEM:FILENAMES_BLOCK);
		int tIndex = tFileNameList.indexOf(aNameFile);
		if (tIndex >= 0) return tIndex;
		tFileNameList.add(aNameFile);
		for (TextureSet tSet : (aIsItem?INSTANCES_ITEM:INSTANCES_BLOCK)) tSet.mList.add(tSet.mIsItem?new TextureSetIconItem(aModID, tSet.mNameSet+"/"+aNameFile):new TextureSetIconBlock(aModID, tSet.mNameSet+"/"+aNameFile));
		return tFileNameList.size() - 1;
	}
	
	public static class TextureSetIconItem implements IIconContainer, Runnable {
		private final String mMod, mName;
		private IIcon mIconColored, mIconOverlay;
		
		public TextureSetIconItem(String aMod, String aName) {
			mName = aName;
			mMod = aMod.toLowerCase();
			GT_API.sItemIconload.add(this);
		}
		
		@Override
		public IIcon getIcon(int aRenderPass) {
			return aRenderPass == 0 ? mIconColored : mIconOverlay;
		}
		
		@Override
		public short[] getIconColor(int aRenderPass) {
			return UNCOLOURED;
		}
		
		@Override
		public int getIconPasses() {
			return 2;
		}
		
		@Override
		public ResourceLocation getTextureFile() {
			return TextureMap.locationItemsTexture;
		}
		
		@Override
		public void registerIcons(IIconRegister aIconRegister) {
			mIconColored = aIconRegister.registerIcon(mMod+":materialicons/"+mName);
			mIconOverlay = aIconRegister.registerIcon(mMod+":materialicons/"+mName+"_OVERLAY");
		}
		
		@Override
		public String toString() {
			return mMod+":materialicons/"+mName;
		}
		
		@Override
		public boolean isUsingColorModulation(int aRenderPass) {
			return aRenderPass == 0;
		}
		
		@Override
		public void run() {
			registerIcons(GT_API.sItemIcons);
		}
	}
	
	public static class TextureSetIconBlock implements IIconContainer, Runnable {
		private final String mMod, mName;
		private IIcon mIconColored, mIconOverlay;
		
		public TextureSetIconBlock(String aMod, String aName) {
			mName = aName;
			mMod = aMod;
			GT_API.sBlockIconload.add(this);
		}
		
		@Override
		public IIcon getIcon(int aRenderPass) {
			return aRenderPass == 0 ? mIconColored : mIconOverlay;
		}
		
		@Override
		public short[] getIconColor(int aRenderPass) {
			return UNCOLOURED;
		}
		
		@Override
		public int getIconPasses() {
			return 2;
		}
		
		@Override
		public ResourceLocation getTextureFile() {
			return TextureMap.locationBlocksTexture;
		}
		
		@Override
		public void registerIcons(IIconRegister aIconRegister) {
			mIconColored = aIconRegister.registerIcon(mMod+":materialicons/"+mName);
			mIconOverlay = aIconRegister.registerIcon(mMod+":materialicons/"+mName+"_OVERLAY");
		}
		
		@Override
		public String toString() {
			return mMod+":materialicons/"+mName;
		}
		
		@Override
		public boolean isUsingColorModulation(int aRenderPass) {
			return aRenderPass == 0;
		}
		
		@Override
		public void run() {
			registerIcons(GT_API.sBlockIcons);
		}
	}
	
	public static final TextureSet
	  SET_BLOCK_NONE				= addTextureSet(ModIDs.GT, F, "NONE")
	, SET_BLOCK_DULL				= addTextureSet(ModIDs.GT, F, "DULL")
	, SET_BLOCK_RUBY				= addTextureSet(ModIDs.GT, F, "RUBY")
	, SET_BLOCK_OPAL				= addTextureSet(ModIDs.GT, F, "OPAL")
	, SET_BLOCK_LEAF				= addTextureSet(ModIDs.GT, F, "LEAF")
	, SET_BLOCK_WOOD				= addTextureSet(ModIDs.GT, F, "WOOD")
	, SET_BLOCK_SAND				= addTextureSet(ModIDs.GT, F, "SAND")
	, SET_BLOCK_FINE				= addTextureSet(ModIDs.GT, F, "FINE")
	, SET_BLOCK_FIERY				= addTextureSet(ModIDs.GT, F, "FIERY")
	, SET_BLOCK_FLUID				= addTextureSet(ModIDs.GT, F, "FLUID")
	, SET_BLOCK_ROUGH				= addTextureSet(ModIDs.GT, F, "ROUGH")
	, SET_BLOCK_STONE				= addTextureSet(ModIDs.GT, F, "STONE")
	, SET_BLOCK_PAPER				= addTextureSet(ModIDs.GT, F, "PAPER")
	, SET_BLOCK_GLASS				= addTextureSet(ModIDs.GT, F, "GLASS")
	, SET_BLOCK_FLINT				= addTextureSet(ModIDs.GT, F, "FLINT")
	, SET_BLOCK_LAPIS				= addTextureSet(ModIDs.GT, F, "LAPIS")
	, SET_BLOCK_SHINY				= addTextureSet(ModIDs.GT, F, "SHINY")
	, SET_BLOCK_SHARDS				= addTextureSet(ModIDs.GT, F, "SHARDS")
	, SET_BLOCK_POWDER				= addTextureSet(ModIDs.GT, F, "POWDER")
	, SET_BLOCK_QUARTZ				= addTextureSet(ModIDs.GT, F, "QUARTZ")
	, SET_BLOCK_EMERALD				= addTextureSet(ModIDs.GT, F, "EMERALD")
	, SET_BLOCK_DIAMOND				= addTextureSet(ModIDs.GT, F, "DIAMOND")
	, SET_BLOCK_LIGNITE				= addTextureSet(ModIDs.GT, F, "LIGNITE")
	, SET_BLOCK_MAGNETIC			= addTextureSet(ModIDs.GT, F, "MAGNETIC")
	, SET_BLOCK_METALLIC			= addTextureSet(ModIDs.GT, F, "METALLIC")
	, SET_BLOCK_NETHERSTAR			= addTextureSet(ModIDs.GT, F, "NETHERSTAR")
	, SET_BLOCK_GEM_VERTICAL		= addTextureSet(ModIDs.GT, F, "GEM_VERTICAL")
	, SET_BLOCK_GEM_HORIZONTAL		= addTextureSet(ModIDs.GT, F, "GEM_HORIZONTAL")
	;
	
	public static final TextureSet
	  SET_ITEMS_NONE				= addTextureSet(ModIDs.GT, T, "NONE")
	, SET_ITEMS_DULL				= addTextureSet(ModIDs.GT, T, "DULL")
	, SET_ITEMS_RUBY				= addTextureSet(ModIDs.GT, T, "RUBY")
	, SET_ITEMS_OPAL				= addTextureSet(ModIDs.GT, T, "OPAL")
	, SET_ITEMS_LEAF				= addTextureSet(ModIDs.GT, T, "LEAF")
	, SET_ITEMS_WOOD				= addTextureSet(ModIDs.GT, T, "WOOD")
	, SET_ITEMS_SAND				= addTextureSet(ModIDs.GT, T, "SAND")
	, SET_ITEMS_FINE				= addTextureSet(ModIDs.GT, T, "FINE")
	, SET_ITEMS_FIERY				= addTextureSet(ModIDs.GT, T, "FIERY")
	, SET_ITEMS_FLUID				= addTextureSet(ModIDs.GT, T, "FLUID")
	, SET_ITEMS_ROUGH				= addTextureSet(ModIDs.GT, T, "ROUGH")
	, SET_ITEMS_STONE				= addTextureSet(ModIDs.GT, T, "STONE")
	, SET_ITEMS_PAPER				= addTextureSet(ModIDs.GT, T, "PAPER")
	, SET_ITEMS_GLASS				= addTextureSet(ModIDs.GT, T, "GLASS")
	, SET_ITEMS_FLINT				= addTextureSet(ModIDs.GT, T, "FLINT")
	, SET_ITEMS_LAPIS				= addTextureSet(ModIDs.GT, T, "LAPIS")
	, SET_ITEMS_SHINY				= addTextureSet(ModIDs.GT, T, "SHINY")
	, SET_ITEMS_SHARDS				= addTextureSet(ModIDs.GT, T, "SHARDS")
	, SET_ITEMS_POWDER				= addTextureSet(ModIDs.GT, T, "POWDER")
	, SET_ITEMS_QUARTZ				= addTextureSet(ModIDs.GT, T, "QUARTZ")
	, SET_ITEMS_EMERALD				= addTextureSet(ModIDs.GT, T, "EMERALD")
	, SET_ITEMS_DIAMOND				= addTextureSet(ModIDs.GT, T, "DIAMOND")
	, SET_ITEMS_LIGNITE				= addTextureSet(ModIDs.GT, T, "LIGNITE")
	, SET_ITEMS_MAGNETIC			= addTextureSet(ModIDs.GT, T, "MAGNETIC")
	, SET_ITEMS_METALLIC			= addTextureSet(ModIDs.GT, T, "METALLIC")
	, SET_ITEMS_NETHERSTAR			= addTextureSet(ModIDs.GT, T, "NETHERSTAR")
	, SET_ITEMS_GEM_VERTICAL		= addTextureSet(ModIDs.GT, T, "GEM_VERTICAL")
	, SET_ITEMS_GEM_HORIZONTAL		= addTextureSet(ModIDs.GT, T, "GEM_HORIZONTAL")
	;
	
	/** Arrays with all Sets, which belong to each other. Shortens the Parameter amount inside MT.java */
	public static final TextureSet[]
	  SET_NONE						= new TextureSet[] {SET_BLOCK_NONE				, SET_ITEMS_NONE			}
	, SET_DULL						= new TextureSet[] {SET_BLOCK_DULL				, SET_ITEMS_DULL			}
	, SET_RUBY						= new TextureSet[] {SET_BLOCK_RUBY				, SET_ITEMS_RUBY			}
	, SET_OPAL						= new TextureSet[] {SET_BLOCK_OPAL				, SET_ITEMS_OPAL			}
	, SET_LEAF						= new TextureSet[] {SET_BLOCK_LEAF				, SET_ITEMS_LEAF			}
	, SET_WOOD						= new TextureSet[] {SET_BLOCK_WOOD				, SET_ITEMS_WOOD			}
	, SET_SAND						= new TextureSet[] {SET_BLOCK_SAND				, SET_ITEMS_SAND			}
	, SET_FINE						= new TextureSet[] {SET_BLOCK_FINE				, SET_ITEMS_FINE			}
	, SET_FIERY						= new TextureSet[] {SET_BLOCK_FIERY				, SET_ITEMS_FIERY			}
	, SET_FLUID						= new TextureSet[] {SET_BLOCK_FLUID				, SET_ITEMS_FLUID			}
	, SET_ROUGH						= new TextureSet[] {SET_BLOCK_ROUGH				, SET_ITEMS_ROUGH			}
	, SET_STONE						= new TextureSet[] {SET_BLOCK_STONE				, SET_ITEMS_STONE			}
	, SET_PAPER						= new TextureSet[] {SET_BLOCK_PAPER				, SET_ITEMS_PAPER			}
	, SET_GLASS						= new TextureSet[] {SET_BLOCK_GLASS				, SET_ITEMS_GLASS			}
	, SET_FLINT						= new TextureSet[] {SET_BLOCK_FLINT				, SET_ITEMS_FLINT			}
	, SET_LAPIS						= new TextureSet[] {SET_BLOCK_LAPIS				, SET_ITEMS_LAPIS			}
	, SET_SHINY						= new TextureSet[] {SET_BLOCK_SHINY				, SET_ITEMS_SHINY			}
	, SET_SHARDS					= new TextureSet[] {SET_BLOCK_SHARDS			, SET_ITEMS_SHARDS			}
	, SET_POWDER					= new TextureSet[] {SET_BLOCK_POWDER			, SET_ITEMS_POWDER			}
	, SET_QUARTZ					= new TextureSet[] {SET_BLOCK_QUARTZ			, SET_ITEMS_QUARTZ			}
	, SET_EMERALD					= new TextureSet[] {SET_BLOCK_EMERALD			, SET_ITEMS_EMERALD			}
	, SET_DIAMOND					= new TextureSet[] {SET_BLOCK_DIAMOND			, SET_ITEMS_DIAMOND			}
	, SET_LIGNITE					= new TextureSet[] {SET_BLOCK_LIGNITE			, SET_ITEMS_LIGNITE			}
	, SET_MAGNETIC					= new TextureSet[] {SET_BLOCK_MAGNETIC			, SET_ITEMS_MAGNETIC		}
	, SET_METALLIC					= new TextureSet[] {SET_BLOCK_METALLIC			, SET_ITEMS_METALLIC		}
	, SET_NETHERSTAR				= new TextureSet[] {SET_BLOCK_NETHERSTAR		, SET_ITEMS_NETHERSTAR		}
	, SET_GEM_VERTICAL				= new TextureSet[] {SET_BLOCK_GEM_VERTICAL		, SET_ITEMS_GEM_VERTICAL	}
	, SET_GEM_HORIZONTAL			= new TextureSet[] {SET_BLOCK_GEM_HORIZONTAL	, SET_ITEMS_GEM_HORIZONTAL	}
	;
}