package gregapi.tileentity;

import static gregapi.data.CS.*;
import gregapi.data.OP;
import gregapi.data.TD;
import gregapi.data.CS.IconsGT;
import gregapi.render.BlockTextureDefault;
import gregapi.render.ITexture;
import net.minecraft.block.Block;
import net.minecraft.nbt.NBTTagCompound;
import net.minecraft.tileentity.TileEntity;

public abstract class TileEntityBase6ConnectorRendered extends TileEntityBase6Connector {
	public float mDiameter = 1.0F;
	public boolean mTransparent = F, mIsGlowing = F;
	
    @Override
	public void readFromNBT2(NBTTagCompound aNBT) {
    	super.readFromNBT2(aNBT);
        if (aNBT.hasKey(NBT_DIAMETER)) mDiameter = Math.max(PIXELS_POS[2], Math.min(PIXELS_NEG[0], (float)aNBT.getDouble(NBT_DIAMETER)));
        if (aNBT.hasKey(NBT_TRANSPARENT)) mTransparent = aNBT.getBoolean(NBT_TRANSPARENT);
    	mIsGlowing = mMaterial.contains(TD.Properties.GLOWING);
    }
    
	@Override
    public int getRenderPasses2(Block aBlock) {
		if (worldObj == null) mConnections = (byte)(SIDE_BITS[SIDE_SOUTH]|SIDE_BITS[SIDE_NORTH]);
		return mDiameter >= 1.0F || mConnections == 0 ? 1 : 7;
	}
    
	@Override
	public boolean setBlockBounds2(Block aBlock, int aRenderPass) {
		if (aRenderPass == 0) {
			if (mDiameter >= 1.0F) return F;
			aBlock.setBlockBounds((1.0F-mDiameter)/2.0F, (1.0F-mDiameter)/2.0F, (1.0F-mDiameter)/2.0F, 1-(1.0F-mDiameter)/2.0F, 1-(1.0F-mDiameter)/2.0F, 1-(1.0F-mDiameter)/2.0F);
			return T;
		}
		// TODO: I need to add the old optimisations back somehow.
		// Even though this Version is way more modular and can adjust to stuff much easier, it does also look bad when rendered with ambient occlusion.
		if (aRenderPass <= 6) {
			DelegatorTileEntity<TileEntity> tDelegator = getAdjacentTileEntity((byte)(aRenderPass - 1), F, F);
			float tDiameter = getConnectorDiameter((byte)(aRenderPass - 1), tDelegator), tLength = getConnectorLength((byte)(aRenderPass - 1), tDelegator);
			switch(aRenderPass - 1) {
			case SIDE_X_NEG: aBlock.setBlockBounds(0-tLength				,   (1.0F-tDiameter)/2.0F	,   (1.0F-tDiameter)/2.0F	,   (1.0F-tDiameter)/2.0F	, 1-(1.0F-tDiameter)/2.0F	, 1-(1.0F-tDiameter)/2.0F	); return T;
			case SIDE_Y_NEG: aBlock.setBlockBounds(  (1.0F-tDiameter)/2.0F	, 0-tLength					,   (1.0F-tDiameter)/2.0F	, 1-(1.0F-tDiameter)/2.0F	,   (1.0F-tDiameter)/2.0F	, 1-(1.0F-tDiameter)/2.0F	); return T;
			case SIDE_Z_NEG: aBlock.setBlockBounds(  (1.0F-tDiameter)/2.0F	,   (1.0F-tDiameter)/2.0F	, 0-tLength					, 1-(1.0F-tDiameter)/2.0F	, 1-(1.0F-tDiameter)/2.0F	,   (1.0F-tDiameter)/2.0F	); return T;
			case SIDE_X_POS: aBlock.setBlockBounds(1-(1.0F-tDiameter)/2.0F	,   (1.0F-tDiameter)/2.0F	,   (1.0F-tDiameter)/2.0F	, 1+tLength					, 1-(1.0F-tDiameter)/2.0F	, 1-(1.0F-tDiameter)/2.0F	); return T;
			case SIDE_Y_POS: aBlock.setBlockBounds(  (1.0F-tDiameter)/2.0F	, 1-(1.0F-tDiameter)/2.0F	,   (1.0F-tDiameter)/2.0F	, 1-(1.0F-tDiameter)/2.0F	, 1+tLength					, 1-(1.0F-tDiameter)/2.0F	); return T;
			case SIDE_Z_POS: aBlock.setBlockBounds(  (1.0F-tDiameter)/2.0F	,   (1.0F-tDiameter)/2.0F	, 1-(1.0F-tDiameter)/2.0F	, 1-(1.0F-tDiameter)/2.0F	, 1-(1.0F-tDiameter)/2.0F	, 1+tLength					); return T;
			}
		}
		return F;
	}
	
	@Override
	public ITexture getTexture2(Block aBlock, int aRenderPass, byte aSide) {
		if (aRenderPass == 0) return mConnections == 0 || (mDiameter >= 1.0F && connected(aSide)) ? getTextureConnected(aSide, mConnections, mDiameter, aRenderPass) : getTextureSide(aSide, mConnections, mDiameter, aRenderPass);
		return connected((byte)(aRenderPass-1)) && aSide != OPPOSITES[aRenderPass-1] ? aSide == aRenderPass-1 && connected(aSide) ? getTextureConnected(aSide, mConnections, mDiameter, aRenderPass) : getTextureSide(aSide, mConnections, mDiameter, aRenderPass) : null;
	}
	
	@Override public int getLightOpacity() {return mTransparent ? mDiameter >= 1.0F ? LIGHT_OPACITY_WATER : mDiameter > 0.5F ? LIGHT_OPACITY_LEAVES : LIGHT_OPACITY_NONE : mDiameter >= 1.0F ? LIGHT_OPACITY_MAX : mDiameter > 0.5F ? LIGHT_OPACITY_WATER : LIGHT_OPACITY_LEAVES;}
	
	@Override public float getSurfaceSize			(byte aSide) {return mDiameter;}
	@Override public float getSurfaceSizeAttachable	(byte aSide) {return mDiameter;}
	@Override public float getSurfaceDistance		(byte aSide) {return connected(aSide)?0.0F:(1.0F-mDiameter)/2.0F;}
	@Override public boolean isSurfaceSolid			(byte aSide) {return mDiameter == 1.0F;}
	@Override public boolean isSurfaceOpaque		(byte aSide) {return mDiameter == 1.0F && !mTransparent;}
	@Override public boolean isSideSolid2			(byte aSide) {return mDiameter == 1.0F;}
	
	public float getConnectorLength(byte aConnectorSide, DelegatorTileEntity<TileEntity> aDelegator) {
		float rLength = 0;
		if (aDelegator.mTileEntity instanceof ITileEntitySurface) rLength = ((ITileEntitySurface)aDelegator.mTileEntity).getSurfaceDistance(aDelegator.mSideOfTileEntity);
		return rLength;
	}
	
	public float getConnectorDiameter(byte aConnectorSide, DelegatorTileEntity<TileEntity> aDelegator) {
		float rDiameter = mDiameter;
		if (aDelegator.mTileEntity instanceof ITileEntitySurface) rDiameter = ((ITileEntitySurface)aDelegator.mTileEntity).getSurfaceSizeAttachable(aDelegator.mSideOfTileEntity);
		// Connect with normal Size when the target cannot connect and ensure that minimum and maximum size are properly done.
		if (rDiameter <= 0.0F || rDiameter > mDiameter) rDiameter = mDiameter; else if (rDiameter < PIXELS_POS[2]) rDiameter = PIXELS_POS[2];
		return rDiameter;
	}
	
	public ITexture getTextureSide		(byte aSide, byte aConnections, float aDiameter, int aRenderPass) {return new BlockTextureDefault(mMaterial, getIconIndexSide		(aSide, aConnections, aDiameter, aRenderPass), STATE_SOLID, mIsGlowing);}
	public ITexture getTextureConnected	(byte aSide, byte aConnections, float aDiameter, int aRenderPass) {return new BlockTextureDefault(mMaterial, getIconIndexConnected	(aSide, aConnections, aDiameter, aRenderPass), STATE_SOLID, mIsGlowing);}
	
	public int getIconIndexSide			(byte aSide, byte aConnections, float aDiameter, int aRenderPass) {return IconsGT.INDEX_BLOCK_PIPE_SIDE;}
	public int getIconIndexConnected	(byte aSide, byte aConnections, float aDiameter, int aRenderPass) {if (aDiameter < 0.37F) return OP.pipeTiny.mIconIndexBlock; if (aDiameter < 0.49F) return OP.pipeSmall.mIconIndexBlock; if (aDiameter < 0.74F) return OP.pipeMedium.mIconIndexBlock; if (aDiameter < 0.99F) return OP.pipeLarge.mIconIndexBlock; return OP.pipeHuge.mIconIndexBlock;}
}