package gregapi.tileentity.connectors;

import static gregapi.data.CS.*;
import gregapi.block.multitileentity.IMultiTileEntity.IMTE_GetCollisionBoundingBoxFromPool;
import gregapi.block.multitileentity.IMultiTileEntity.IMTE_OnEntityCollidedWithBlock;
import gregapi.code.ArrayListNoNulls;
import gregapi.code.TagData;
import gregapi.data.CS.IconsGT;
import gregapi.data.*;
import gregapi.data.LH.Chat;
import gregapi.log.GT_Log;
import gregapi.old.Textures;
import gregapi.render.BlockTextureDefault;
import gregapi.render.BlockTextureMulti;
import gregapi.render.ITexture;
import gregapi.tileentity.*;
import gregapi.util.UT;

import java.util.ArrayList;
import java.util.Collection;
import java.util.List;

import net.minecraft.entity.Entity;
import net.minecraft.item.ItemStack;
import net.minecraft.nbt.NBTTagCompound;
import net.minecraft.tileentity.TileEntity;
import net.minecraft.util.AxisAlignedBB;
import net.minecraftforge.common.util.ForgeDirection;
import net.minecraftforge.fluids.FluidStack;
import net.minecraftforge.fluids.FluidTank;
import net.minecraftforge.fluids.IFluidHandler;
import net.minecraftforge.fluids.IFluidTank;

public class MultiTileEntityPipeFluid extends TileEntityBase6ConnectorRendered implements IFluidHandler, ITileEntityGibbl, ITileEntityTemperature, ITileEntityProgress, IMTE_GetCollisionBoundingBoxFromPool, IMTE_OnEntityCollidedWithBlock {
	private static double HEAT_RESISTANCE_BONUS = 1.25;
	
	public byte mLastReceivedFrom = 0, mRenderType = 0;
	public long mTemperature = DEFAULT_ENVIRONMENT_TEMPERATURE, mTransferredAmount = 0;
	public boolean mGasProof;
	public FluidTank mTank = new FluidTank(1000);
	
	@Override
	public void readFromNBT2(NBTTagCompound aNBT) {
    	super.readFromNBT2(aNBT);
    	if (aNBT.hasKey("gt.mlast")) mLastReceivedFrom = aNBT.getByte("gt.mlast");
    	if (aNBT.hasKey("gt.mtransfer")) mTransferredAmount = aNBT.getLong("gt.mtransfer");
    	if (aNBT.hasKey(NBT_PIPERENDER)) mRenderType = aNBT.getByte(NBT_PIPERENDER);
    	if (aNBT.hasKey(NBT_GASPROOF)) mGasProof = aNBT.getBoolean(NBT_GASPROOF);
    	if (aNBT.hasKey(NBT_CAPACITY)) mTank.setCapacity(aNBT.getInteger(NBT_CAPACITY) * 2);
        if (aNBT.hasKey(NBT_TANK)) mTank.readFromNBT(aNBT.getCompoundTag(NBT_TANK));
	}
	
	@Override
	public void writeToNBT2(NBTTagCompound aNBT) {
    	super.writeToNBT2(aNBT);
		aNBT.setByte("gt.mlast", mLastReceivedFrom);
        aNBT.setTag(NBT_TANK, mTank.writeToNBT(new NBTTagCompound()));
        UT.NBT.setNumber(aNBT, "gt.mtransfer", mTransferredAmount);
	}
	
	@Override
	public void addToolTips(List aList, ItemStack aStack, boolean aF3_H) {
		aList.add(Chat.CYAN		+ LH.get(LH.PIPE_STATS_BANDWIDTH) + (mTank.getCapacity()/4) + " L/t");
		aList.add(Chat.DRED		+ LH.get(LH.HAZARD_MELTDOWN) + " (" + getTemperatureMax(SIDE_ANY) + " K)");
		super.addToolTips(aList, aStack, aF3_H);
	}
	
    @Override
    public void onTick(long aTimer, boolean aIsServerSide) {
    	super.onTick(aTimer, aIsServerSide);
    	
	    if (aIsServerSide && ((aTimer % 2 == 0) != UT.Worlds.areCoordsEven(this))) {
	    	mTransferredAmount = 0;
	    	
	    	FluidStack tFluid = mTank.getFluid();
	    	
	    	if (mTank.getFluidAmount() > 0) {
	    		mTemperature = tFluid.getFluid().getTemperature(tFluid);
	    		if (!mGasProof && tFluid.getFluid().isGaseous(tFluid)) {
	    			mTransferredAmount += Math.min(8, mTank.getFluidAmount());
	    			mTank.drain(8, T);
//	    			sendSound((byte)9);
	    			try {for (Entity tEntity : (ArrayList<Entity>)worldObj.getEntitiesWithinAABB(Entity.class, AxisAlignedBB.getBoundingBox(xCoord-2, yCoord-2, zCoord-2, xCoord+3, yCoord+3, zCoord+3))) UT.Entities.applyTemperatureDamage(tEntity, mTemperature, 2.0F);} catch(Throwable e) {if (D1) e.printStackTrace(GT_Log.err);}
	    		}
	    	} else {
	    		mTemperature = UT.Worlds.getEnvironmentalTemperature(worldObj, xCoord, yCoord, zCoord);
	    	}
	    	
    		if (mTemperature > getTemperatureMax(SIDE_ANY)) {setOnFire(); if (getRandomNumber(100) == 0) {setToFire(); return;}}
    		
			distribute(ALL_SIDES_VALID);
			
			mLastReceivedFrom = 0;
		}
    }
    
    public void distribute(byte[] aSides) {
    	ArrayListNoNulls<DelegatorTileEntity<IFluidHandler>> tAdjacentTanks = new ArrayListNoNulls<DelegatorTileEntity<IFluidHandler>>(), tAdjacentPipes = new ArrayListNoNulls<DelegatorTileEntity<IFluidHandler>>();
		DelegatorTileEntity<IFluidHandler> tTank;
		
		long tAmount = mTank.getFluidAmount();
		if (tAmount <= 0) return;
		byte tPipeCount = 1;
		
    	for (byte aSide : aSides) if (canEmitFluidsTo(aSide) && !FACE_CONNECTED[aSide][mLastReceivedFrom]) {
			tTank = getAdjacentTank(aSide);
			if (tTank.mTileEntity != null) {
				if (tTank.mTileEntity instanceof MultiTileEntityPipeFluid) {
					if (((MultiTileEntityPipeFluid)tTank.mTileEntity).mTank.getFluidAmount() < mTank.getFluidAmount()) {
						tAmount += ((MultiTileEntityPipeFluid)tTank.mTileEntity).mTank.getFluidAmount();
						tPipeCount++;
						tAdjacentTanks.add(tTank);
					}
				} else if (tTank.mTileEntity.fill(tTank.getForgeSideOfTileEntity(), mTank.getFluid(), F) > 0) {
					tAdjacentTanks.add(tTank);
				}
			}
		}
    	
    	if (tAmount % tPipeCount == 0) {
        	tAmount /= tPipeCount;
    	} else {
        	tAmount /= tPipeCount;
        	tAmount++;
    	}
    	
	   	if (tAmount > 0) for (int i = tAdjacentTanks.size(); i > 0 && mTank.getFluidAmount() > 0;) {
	    	tTank = tAdjacentTanks.get(--i);
		    if (tTank.mTileEntity instanceof MultiTileEntityPipeFluid) {
		    	tAdjacentTanks.remove(i);
		    	tAdjacentPipes.add(tTank);
		    	int tDrained = tTank.mTileEntity.fill(tTank.getForgeSideOfTileEntity(), mTank.drain(UT.Code.bindInt(tAmount-((MultiTileEntityPipeFluid)tTank.mTileEntity).mTank.getFluidAmount()), F), T);
				mTransferredAmount += tDrained;
		    	mTank.drain(tDrained, T);
		    }
	    }
	   	
		if (!tAdjacentTanks.isEmpty()) {
			tAmount = mTank.getFluidAmount() / tAdjacentTanks.size();
			if (tAmount <= 0) {
				while (mTank.getFluidAmount() > 0 && !tAdjacentTanks.isEmpty()) {
					tAdjacentTanks.remove(tTank = tAdjacentTanks.get(getRandomNumber(tAdjacentTanks.size())));
					int tDrained = tTank.mTileEntity.fill(tTank.getForgeSideOfTileEntity(), mTank.drain(1, F), T);
					mTransferredAmount += tDrained;
					mTank.drain(tDrained, T);
				}
			} else {
				for (DelegatorTileEntity<IFluidHandler> tTank2 : tAdjacentTanks) {
					int tDrained = tTank2.mTileEntity.fill(tTank2.getForgeSideOfTileEntity(), mTank.drain(UT.Code.bindInt(tAmount), F), T);
					mTransferredAmount += tDrained;
					mTank.drain(tDrained, T);
				}
			}
		}
		
		if (!tAdjacentPipes.isEmpty() && mTank.getFluidAmount() > mTank.getCapacity() / 2) {
			tAmount = (mTank.getFluidAmount() - mTank.getCapacity() / 2) / tAdjacentPipes.size();
			if (tAmount > 0) {
				for (DelegatorTileEntity<IFluidHandler> tTank2 : tAdjacentPipes) {
					int tDrained = tTank2.mTileEntity.fill(tTank2.getForgeSideOfTileEntity(), mTank.drain(UT.Code.bindInt(tAmount), F), T);
					mTransferredAmount += tDrained;
					mTank.drain(tDrained, T);
				}
			}
		}
    }
    
	@Override
	public void onEntityCollidedWithBlock(Entity aEntity) {
		UT.Entities.applyTemperatureDamage(aEntity, mTemperature, 1.0F);
	}
	
	@Override protected IFluidTank getFluidTankFillable(byte aSide, FluidStack aFluidToFill) {if (canAcceptFluidsFrom(aSide)) return mTank; return null;}
	@Override protected IFluidTank getFluidTankDrainable(byte aSide, FluidStack aFluidToDrain) {if (canEmitFluidsTo(aSide)) return mTank; return null;}
	@Override protected IFluidTank[] getFluidTanks(byte aSide) {return new IFluidTank[] {mTank};}
	@Override public int fill(ForgeDirection aDirection, FluidStack aFluid, boolean aDoFill) {if (aDoFill) mLastReceivedFrom |= SIDE_BITS[UT.Code.side(aDirection)]; return super.fill(aDirection, aFluid, aDoFill);}
	
	@Override public AxisAlignedBB getCollisionBoundingBoxFromPool() {return AxisAlignedBB.getBoundingBox(xCoord+PIXELS_POS[2], yCoord+PIXELS_POS[2], zCoord+PIXELS_POS[2], xCoord+PIXELS_NEG[2], yCoord+PIXELS_NEG[2], zCoord+PIXELS_NEG[2]);}
    
	public boolean canEmitFluidsTo							(byte aSide) {return connected(aSide);}
	public boolean canAcceptFluidsFrom						(byte aSide) {return connected(aSide);}
	
	@Override public boolean canConnect						(byte aSide, DelegatorTileEntity<TileEntity> aDelegator) {return aDelegator.mTileEntity instanceof IFluidHandler && UT.Code.exists(0, ((IFluidHandler)aDelegator.mTileEntity).getTankInfo(aDelegator.getForgeSideOfTileEntity()));}
	
	@Override public long getGibblValue						(byte aSide) {return mTank.getFluidAmount();}
	@Override public long getGibblMax						(byte aSide) {return mTank.getCapacity();}
	
	@Override public long getProgressValue					(byte aSide) {return mTransferredAmount / 2;}
	@Override public long getProgressMax					(byte aSide) {return mTank.getCapacity() / 2;}
	
	@Override public long getTemperatureValue				(byte aSide) {return mTemperature;}
	@Override public long getTemperatureMax					(byte aSide) {return (long)(mMaterial.mMeltingPoint * HEAT_RESISTANCE_BONUS);}
	
	@Override public ITexture getTextureSide				(byte aSide, byte aConnections, float aDiameter, int aRenderPass) {BlockTextureDefault tBase = new BlockTextureDefault(mMaterial, getIconIndexSide		(aSide, aConnections, aDiameter, aRenderPass), STATE_SOLID, mIsGlowing); switch(mRenderType) {case 1: return new BlockTextureMulti(tBase, new BlockTextureDefault(Textures.BlockIcons.PIPE_RESTRICTOR)); default: return tBase;}}
	@Override public ITexture getTextureConnected			(byte aSide, byte aConnections, float aDiameter, int aRenderPass) {BlockTextureDefault tBase = new BlockTextureDefault(mMaterial, getIconIndexConnected	(aSide, aConnections, aDiameter, aRenderPass), STATE_SOLID, mIsGlowing); switch(mRenderType) {case 1: return new BlockTextureMulti(tBase, new BlockTextureDefault(Textures.BlockIcons.PIPE_RESTRICTOR)); default: return tBase;}}
	
	@Override public int getIconIndexSide					(byte aSide, byte aConnections, float aDiameter, int aRenderPass) {return IconsGT.INDEX_BLOCK_PIPE_SIDE;}
	@Override public int getIconIndexConnected				(byte aSide, byte aConnections, float aDiameter, int aRenderPass) {if (aDiameter < 0.37F) return OP.pipeTiny.mIconIndexBlock; if (aDiameter < 0.49F) return OP.pipeSmall.mIconIndexBlock; if (aDiameter < 0.74F) return OP.pipeMedium.mIconIndexBlock; if (aDiameter < 0.99F) return OP.pipeLarge.mIconIndexBlock; return OP.pipeHuge.mIconIndexBlock;}
	
	@Override public Collection<TagData> getConnectorTypes	(byte aSide) {return TD.Connectors.ALL_PIPE_FLUID;}
	
	@Override public String getTileEntityName() {return "gt.multitileentity.connector.pipe.fluid";}
}