package gregapi.tileentity.connectors;

import static gregapi.data.CS.*;
import gregapi.code.ArrayListNoNulls;
import gregapi.code.TagData;
import gregapi.data.CS.IconsGT;
import gregapi.data.*;
import gregapi.data.LH.Chat;
import gregapi.old.Textures;
import gregapi.render.BlockTextureDefault;
import gregapi.render.BlockTextureMulti;
import gregapi.render.ITexture;
import gregapi.tileentity.*;
import gregapi.util.UT;

import java.util.ArrayList;
import java.util.Collection;
import java.util.HashMap;
import java.util.List;

import net.minecraft.inventory.IInventory;
import net.minecraft.inventory.ISidedInventory;
import net.minecraft.item.ItemStack;
import net.minecraft.nbt.NBTTagCompound;
import net.minecraft.tileentity.TileEntity;
import net.minecraft.tileentity.TileEntityDispenser;
import net.minecraft.tileentity.TileEntityHopper;

public class MultiTileEntityPipeItem extends TileEntityBase6ConnectorRendered implements ISidedInventory, ITileEntityProgress, ITileEntityItemPipe {
	public long mTransferredItems = 0, mStepSize = 1;
	public byte mLastReceivedFrom = SIDE_UNDEFINED, oLastReceivedFrom = SIDE_UNDEFINED, mRenderType = 0;
	
	@Override
	public void readFromNBT2(NBTTagCompound aNBT) {
    	super.readFromNBT2(aNBT);
    	if (aNBT.hasKey("gt.olast")) oLastReceivedFrom = aNBT.getByte("gt.olast");
    	if (aNBT.hasKey("gt.mlast")) mLastReceivedFrom = aNBT.getByte("gt.mlast");
    	if (aNBT.hasKey("gt.mtransfer")) mTransferredItems = aNBT.getLong("gt.mtransfer");
    	if (aNBT.hasKey(NBT_PIPESTEPSIZE)) mStepSize = aNBT.getLong(NBT_PIPESTEPSIZE);
    	if (aNBT.hasKey(NBT_PIPERENDER)) mRenderType = aNBT.getByte(NBT_PIPERENDER);
	}
	
	@Override
	public void writeToNBT2(NBTTagCompound aNBT) {
    	super.writeToNBT2(aNBT);
		aNBT.setByte("gt.olast", oLastReceivedFrom);
		aNBT.setByte("gt.mlast", mLastReceivedFrom);
		UT.NBT.setNumber(aNBT, "gt.mtransfer", mTransferredItems);
	}
	
	@Override
	public void addToolTips(List aList, ItemStack aStack, boolean aF3_H) {
		aList.add(Chat.CYAN + LH.get(LH.PIPE_STATS_STEPSIZE) + mStepSize);
		aList.add(Chat.CYAN + LH.get(LH.PIPE_STATS_BANDWIDTH) + getPipeCapacity());
		super.addToolTips(aList, aStack, aF3_H);
	}
	
    @Override
    public void onTick(long aTimer, boolean aIsServerSide) {
    	super.onTick(aTimer, aIsServerSide);
    	
	    if (aIsServerSide && aTimer % 10 == 0) {
			if (aTimer % 20 == 0) mTransferredItems = 0;
			
	    	if (oLastReceivedFrom == mLastReceivedFrom) {
        		ArrayList<ITileEntityItemPipe> tPipeList = new ArrayListNoNulls<ITileEntityItemPipe>();
	        	for (boolean temp = T; temp && UT.Code.containsSomething(getInventory()) && pipeCapacityCheck();) {
	        		temp = F;
	        		tPipeList.clear();
	        		for (ITileEntityItemPipe tTileEntity : UT.Code.sortByValuesAcending(ITileEntityItemPipe.Util.scanPipes(this, new HashMap<ITileEntityItemPipe, Long>(), 0, F, F)).keySet()) {
	        			if (temp) break;
	        			tPipeList.add(tTileEntity);
	        			while (!temp && UT.Code.containsSomething(getInventory()) && tTileEntity.sendItemStack(this)) for (ITileEntityItemPipe tPipe : tPipeList) if (!tPipe.incrementTransferCounter(1)) temp = T;
	        		}
	        	}
	    	}
	    	
			if (!UT.Code.containsSomething(getInventory())) mLastReceivedFrom = SIDE_UNDEFINED;
	    	oLastReceivedFrom = mLastReceivedFrom;
		}
    }
    
	@Override
	public boolean insertItemStackIntoTileEntity(Object aSender, byte aSide) {
    	if (canEmitItemsTo(aSide, aSender)) {
	    	DelegatorTileEntity<TileEntity> tDelegator = getAdjacentTileEntity(aSide);
			if (tDelegator.mTileEntity != null && !(tDelegator.mTileEntity instanceof TileEntityBase6Connector)) {
				if ((!(tDelegator.mTileEntity instanceof TileEntityHopper) && !(tDelegator.mTileEntity instanceof TileEntityDispenser)) || getMetaDataAtSide(aSide) != tDelegator.mSideOfTileEntity) {
					return UT.Inventories.moveOneItemStack(aSender, tDelegator, SIDE_ANY, tDelegator.mSideOfTileEntity, null, F, (byte)64, (byte)1, (byte)64, (byte)1) > 0;
				}
			}
    	}
    	return F;
    }
	
	@Override public boolean incrementTransferCounter(long aIncrement) {mTransferredItems += aIncrement; return pipeCapacityCheck();}
	@Override public boolean sendItemStack(Object aSender) {if (pipeCapacityCheck()) for (byte i = 0, j = (byte)getRandomNumber(6); i < 6; i++) if (insertItemStackIntoTileEntity(aSender, (byte)((i+j)%6))) return T; return F;}
    
	@Override public boolean pipeCapacityCheck() {return mTransferredItems <= 0 || getPipeContent() < getMaxPipeCapacity();}
    @Override public long getStepSize() {return mStepSize;}
    
	protected long getPipeContent() {return mTransferredItems;}
	protected long getMaxPipeCapacity() {return Math.max(1, getPipeCapacity());}
	protected long getPipeCapacity() {return getSizeInventory();}
	
	protected int[] ACCESSIBLE_SLOTS;
	@Override public int[] getAccessibleSlotsFromSide(int aSide) {return ACCESSIBLE_SLOTS;}
	@Override public boolean canDrop(int aInventorySlot) {return T;}
	@Override public boolean canInsertItem(int aSlot, ItemStack aStack, int aSide) {if (!UT.Code.containsSomething(getInventory())) mLastReceivedFrom = (byte)aSide; return mLastReceivedFrom == aSide && slot(aSlot) == null;}
	@Override public boolean canExtractItem(int aSlot, ItemStack aStack, int aSide) {return T;}
	@Override public ItemStack[] getDefaultInventory(NBTTagCompound aNBT) {ItemStack[] rStack = super.getDefaultInventory(aNBT); ACCESSIBLE_SLOTS = new int[rStack.length]; for (int i = 0; i < ACCESSIBLE_SLOTS.length; i++) ACCESSIBLE_SLOTS[i] = i; return rStack;}
	
	@Override public boolean canEmitItemsTo					(byte aSide, Object aSender) {return (aSender != this || aSide != mLastReceivedFrom) && connected(aSide);}
	@Override public boolean canAcceptItemsFrom				(byte aSide, Object aSender) {return connected(aSide);}
	
	@Override public boolean canConnect						(byte aSide, DelegatorTileEntity<TileEntity> aDelegator) {return (aDelegator.mTileEntity instanceof ISidedInventory ? ((ISidedInventory)aDelegator.mTileEntity).getAccessibleSlotsFromSide(aDelegator.mSideOfTileEntity).length > 0 : (aDelegator.mTileEntity instanceof IInventory && ((IInventory)aDelegator.mTileEntity).getSizeInventory() > 0)) || UT.Inventories.isConnectableNonInventoryPipe(aDelegator.mTileEntity, aDelegator.mSideOfTileEntity);}
	
	@Override public long getProgressValue					(byte aSide) {return getPipeContent()*64;}
	@Override public long getProgressMax					(byte aSide) {return getMaxPipeCapacity()*64;}
	
	@Override public ITexture getTextureSide				(byte aSide, byte aConnections, float aDiameter, int aRenderPass) {BlockTextureDefault tBase = new BlockTextureDefault(mMaterial, getIconIndexSide		(aSide, aConnections, aDiameter, aRenderPass), STATE_SOLID, mIsGlowing); switch(mRenderType) {case 1: return new BlockTextureMulti(tBase, new BlockTextureDefault(Textures.BlockIcons.PIPE_RESTRICTOR)); default: return tBase;}}
	@Override public ITexture getTextureConnected			(byte aSide, byte aConnections, float aDiameter, int aRenderPass) {BlockTextureDefault tBase = new BlockTextureDefault(mMaterial, getIconIndexConnected	(aSide, aConnections, aDiameter, aRenderPass), STATE_SOLID, mIsGlowing); switch(mRenderType) {case 1: return new BlockTextureMulti(tBase, new BlockTextureDefault(Textures.BlockIcons.PIPE_RESTRICTOR)); default: return tBase;}}
	
	@Override public int getIconIndexSide					(byte aSide, byte aConnections, float aDiameter, int aRenderPass) {return IconsGT.INDEX_BLOCK_PIPE_SIDE;}
	@Override public int getIconIndexConnected				(byte aSide, byte aConnections, float aDiameter, int aRenderPass) {if (aDiameter < 0.37F) return OP.pipeTiny.mIconIndexBlock; if (aDiameter < 0.49F) return OP.pipeSmall.mIconIndexBlock; if (aDiameter < 0.74F) return OP.pipeMedium.mIconIndexBlock; if (aDiameter < 0.99F) return OP.pipeLarge.mIconIndexBlock; return OP.pipeHuge.mIconIndexBlock;}
	
	@Override public Collection<TagData> getConnectorTypes	(byte aSide) {return TD.Connectors.ALL_PNEUMATIC_ITEM;}
	
	@Override public String getTileEntityName() {return "gt.multitileentity.connector.pipe.item";}
}